/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2009 by Nakata, Maho
 * 
 * $Id: mblas_gmp.h,v 1.8 2009/09/17 00:59:02 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#ifndef _MBLAS_GMP_H_
#define _MBLAS_GMP_H_

#include <mpack_config.h>
#include <gmpxx.h>
#include <mpc_class.h>
#include <mutils_gmp.h>
 
#if !defined __MPACK_ERRNO__
#define _MPACK_EXTERN_ extern
#else
#define _MPACK_EXTERN_
#endif

_MPACK_EXTERN_ int mpack_errno;

/* LEVEL 1 MBLAS */
void Rrot(mpackint n, mpf_class * dx, mpackint incx, mpf_class * dy,
    mpackint incy, mpf_class c, mpf_class s);
void Rrotm(mpackint n, mpf_class * x, mpackint incx, mpf_class * y,
    mpackint incy, mpf_class * dparam);
void CRrot(mpackint n, mpc_class * cx, mpackint incx, mpc_class * cy,
    mpackint incy, mpf_class c, mpf_class s);
void Cswap(mpackint n, mpc_class * cx, mpackint incx, mpc_class * cy,
    mpackint incy);
void Rswap(mpackint n, mpf_class * dx, mpackint incx, mpf_class * dy,
    mpackint incy);
void Cscal(mpackint n, mpc_class ca, mpc_class * cx, mpackint incx);
void CRscal(mpackint n, mpf_class sa, mpc_class * cx, mpackint incx);
void Rscal(mpackint n, mpf_class ca, mpf_class * cx, mpackint incx);
void Ccopy(mpackint n, mpc_class * cx, mpackint incx, mpc_class * cy,
    mpackint incy);
void Crotg(mpc_class * ca, mpc_class cb, mpf_class * c, mpc_class * s);
void Rrotg(mpf_class * da, mpf_class * db, mpf_class * c, mpf_class * s);
void Rcopy(mpackint n, mpf_class * dx, mpackint incx, mpf_class * dy,
    mpackint incy);
void Raxpy(mpackint n, mpf_class da, mpf_class * dx, mpackint incx, mpf_class * dy, mpackint incy);
void Caxpy(mpackint n, mpc_class ca, mpc_class * cx, mpackint incx,
    mpc_class * cy, mpackint incy);
mpf_class Rdot(mpackint n, mpf_class * dx, mpackint incx, mpf_class * dy,
    mpackint incy);
mpc_class Cdotc(mpackint n, mpc_class * cx, mpackint incx, mpc_class * cy,
    mpackint incy);
mpc_class Cdotu(mpackint n, mpc_class * cx, mpackint incx, mpc_class * cy,
    mpackint incy);
mpf_class RCnrm2(mpackint n, mpc_class * x, mpackint incx);
mpf_class Rnrm2(mpackint n, mpf_class * x, mpackint incx);
mpf_class RCasum(mpackint n, mpc_class * zx, mpackint incx);
mpf_class Rasum(mpackint n, mpf_class * dx, mpackint incx);
mpackint iCamax(mpackint n, mpc_class * dx, mpackint incx);
mpackint iRamax(mpackint n, mpf_class * dx, mpackint incx);
mpf_class RCabs1(mpc_class z);
int Mlsame_gmp(const char *a, const char *b);
void Mxerbla_gmp(const char *srname, int info);

/* LEVEL 2 MBLAS */
void Cgemv(const char *trans, mpackint m, mpackint n, mpc_class alpha,
    mpc_class * A, mpackint lda, mpc_class * x, mpackint incx, mpc_class beta,
    mpc_class * y, mpackint incy);
void Rgemv(const char *trans, mpackint m, mpackint n, mpf_class alpha,
    mpf_class * A, mpackint lda, mpf_class * x, mpackint incx, mpf_class beta,
    mpf_class * y, mpackint incy);
void Cgbmv(const char *trans, mpackint m, mpackint n, mpackint kl, mpackint ku,
    mpc_class alpha, mpc_class * A, mpackint lda, mpc_class * x, mpackint incx,
    mpc_class beta, mpc_class * y, mpackint incy);
void Rgbmv(const char *trans, mpackint m, mpackint n, mpackint kl, mpackint ku,
    mpf_class alpha, mpf_class * A, mpackint lda, mpf_class * x, mpackint incx,
    mpf_class beta, mpf_class * y, mpackint incy);
void Chemv(const char *uplo, mpackint n, mpc_class alpha, mpc_class * A,
    mpackint lda, mpc_class * x, mpackint incx, mpc_class beta, mpc_class * y,
    mpackint incy);
void Chbmv(const char *uplo, mpackint n, mpackint k, mpc_class alpha,
    mpc_class * A, mpackint lda, mpc_class * x, mpackint incx, mpc_class beta,
    mpc_class * y, mpackint incy);
void Chpmv(const char *uplo, mpackint n, mpc_class alpha, mpc_class * AP,
    mpc_class * x, mpackint incx, mpc_class beta, mpc_class * y,
    mpackint incy);
void Rsymv(const char *uplo, mpackint n, mpf_class alpha, mpf_class * A,
    mpackint lda, mpf_class * x, mpackint incx, mpf_class beta, mpf_class * y,
    mpackint incy);
void Rsbmv(const char *uplo, mpackint n, mpackint k, mpf_class alpha,
    mpf_class * A, mpackint lda, mpf_class * x, mpackint incx, mpf_class beta,
    mpf_class * y, mpackint incy);
void Rspmv(const char *uplo, mpackint n, mpf_class alpha, mpf_class * AP,
    mpf_class * x, mpackint incx, mpf_class beta, mpf_class * y,
    mpackint incy);
void Ctrmv(const char *uplo, const char *trans, const char *diag, mpackint n,
    mpc_class * A, mpackint lda, mpc_class * x, mpackint incx);
void Rtrmv(const char *uplo, const char *trans, const char *diag, mpackint n,
    mpf_class * A, mpackint lda, mpf_class * x, mpackint incx);
void Ctbmv(const char *uplo, const char *trans, const char *diag, mpackint n,
    mpackint k, mpc_class * A, mpackint lda, mpc_class * x, mpackint incx);
void Rtbmv(const char *uplo, const char *trans, const char *diag, mpackint n,
    mpackint k, mpf_class * A, mpackint lda, mpf_class * x, mpackint incx);
void Ctpmv(const char *uplo, const char *trans, const char *diag, mpackint n,
    mpc_class * AP, mpc_class * x, mpackint incx);
void Rtpmv(const char *uplo, const char *trans, const char *diag, mpackint n,
    mpf_class * AP, mpf_class * x, mpackint incx);
void Ctrsv(const char *uplo, const char *trans, const char *diag, mpackint n,
    mpc_class * A, mpackint lda, mpc_class * x, mpackint incx);
void Rtrsv(const char *uplo, const char *trans, const char *diag, mpackint n,
    mpf_class * A, mpackint lda, mpf_class * x, mpackint incx);
void Ctbsv(const char *uplo, const char *trans, const char *diag, mpackint n,
    mpackint k, mpc_class * A, mpackint lda, mpc_class * x, mpackint incx);
void Rtbsv(const char *uplo, const char *trans, const char *diag, mpackint n,
    mpackint k, mpf_class * A, mpackint lda, mpf_class * x, mpackint incx);
void Ctpsv(const char *uplo, const char *trans, const char *diag, mpackint n,
    mpc_class * AP, mpc_class * x, mpackint incx);
void Rtpsv(const char *uplo, const char *trans, const char *diag, mpackint n,
    mpf_class * AP, mpf_class * x, mpackint incx);
void Rger(mpackint m, mpackint n, mpf_class alpha, mpf_class * x,
    mpackint incx, mpf_class * y, mpackint incy, mpf_class * A, mpackint lda);
void Cgeru(mpackint m, mpackint n, mpc_class alpha, mpc_class * x,
    mpackint incx, mpc_class * y, mpackint incy, mpc_class * A, mpackint lda);
void Cgerc(mpackint m, mpackint n, mpc_class alpha, mpc_class * x,
    mpackint incx, mpc_class * y, mpackint incy, mpc_class * A, mpackint lda);
void Cher(const char *uplo, mpackint n, mpf_class alpha, mpc_class * x,
    mpackint incx, mpc_class * A, mpackint lda);
void Chpr(const char *uplo, mpackint n, mpf_class alpha, mpc_class * x,
    mpackint incx, mpc_class * AP);
void Cher2(const char *uplo, mpackint n, mpc_class alpha, mpc_class * x,
    mpackint incx, mpc_class * y, mpackint incy, mpc_class * A, mpackint lda);
void Chpr2(const char *uplo, mpackint n, mpc_class alpha, mpc_class * x,
    mpackint incx, mpc_class * y, mpackint incy, mpc_class * AP);
void Rsyr(const char *uplo, mpackint n, mpf_class alpha, mpf_class * x,
    mpackint incx, mpf_class * A, mpackint lda);
void Rspr(const char *uplo, mpackint n, mpf_class alpha, mpf_class * x,
    mpackint incx, mpf_class * AP);
void Rsyr2(const char *uplo, mpackint n, mpf_class alpha, mpf_class * x,
    mpackint incx, mpf_class * y, mpackint incy, mpf_class * A, mpackint lda);
void Rspr2(const char *uplo, mpackint n, mpf_class alpha, mpf_class * x,
    mpackint incx, mpf_class * y, mpackint incy, mpf_class * AP);

/* LEVEL 3 MBLAS */
void Cgemm(const char *transa, const char *transb, mpackint m, mpackint n,
    mpackint k, mpc_class alpha, mpc_class * A, mpackint lda, mpc_class * B,
    mpackint ldb, mpc_class beta, mpc_class * C, mpackint ldc);
void Rgemm(const char *transa, const char *transb, mpackint m, mpackint n,
    mpackint k, mpf_class alpha, mpf_class * A, mpackint lda, mpf_class * B,
    mpackint ldb, mpf_class beta, mpf_class * C, mpackint ldc);
void Csymm(const char *side, const char *uplo, mpackint m, mpackint n,
    mpc_class alpha, mpc_class * A, mpackint lda, mpc_class * B, mpackint ldb,
    mpc_class beta, mpc_class * C, mpackint ldc);
void Rsymm(const char *side, const char *uplo, mpackint m, mpackint n,
    mpf_class alpha, mpf_class * A, mpackint lda, mpf_class * B, mpackint ldb,
    mpf_class beta, mpf_class * C, mpackint ldc);
void Chemm(const char *side, const char *uplo, mpackint m, mpackint n,
    mpc_class alpha, mpc_class * A, mpackint lda, mpc_class * B, mpackint ldb,
    mpc_class beta, mpc_class * C, mpackint ldc);
void Csyrk(const char *uplo, const char *trans, mpackint n, mpackint k,
    mpc_class alpha, mpc_class * A, mpackint lda, mpc_class beta,
    mpc_class * C, mpackint ldc);
void Rsyrk(const char *uplo, const char *trans, mpackint n, mpackint k,
    mpf_class alpha, mpf_class * A, mpackint lda, mpf_class beta,
    mpf_class * C, mpackint ldc);
void Cherk(const char *uplo, const char *trans, mpackint n, mpackint k,
    mpf_class alpha, mpc_class * A, mpackint lda, mpf_class beta,
    mpc_class * C, mpackint ldc);
void Csyr2k(const char *uplo, const char *trans, mpackint n, mpackint k,
    mpc_class alpha, mpc_class * A, mpackint lda, mpc_class * B, mpackint ldb,
    mpc_class beta, mpc_class * C, mpackint ldc);
void Rsyr2k(const char *uplo, const char *trans, mpackint n, mpackint k,
    mpf_class alpha, mpf_class * A, mpackint lda, mpf_class * B, mpackint ldb,
    mpf_class beta, mpf_class * C, mpackint ldc);
void Cher2k(const char *uplo, const char *trans, mpackint n, mpackint k,
    mpc_class alpha, mpc_class * A, mpackint lda, mpc_class * B, mpackint ldb,
    mpf_class beta, mpc_class * C, mpackint ldc);
void Ctrmm(const char *side, const char *uplo, const char *transa,
    const char *diag, mpackint m, mpackint n, mpc_class alpha, mpc_class * A,
    mpackint lda, mpc_class * B, mpackint ldb);
void Rtrmm(const char *side, const char *uplo, const char *transa,
    const char *diag, mpackint m, mpackint n, mpf_class alpha, mpf_class * A,
    mpackint lda, mpf_class * B, mpackint ldb);
void Ctrsm(const char *side, const char *uplo, const char *transa,
    const char *diag, mpackint m, mpackint n, mpc_class alpha, mpc_class * A,
    mpackint lda, mpc_class * B, mpackint ldb);
void Rtrsm(const char *side, const char *uplo, const char *transa,
    const char *diag, mpackint m, mpackint n, mpf_class alpha, mpf_class * A,
    mpackint lda, mpf_class * B, mpackint ldb);

#endif
