/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: mpack_debug.h,v 1.29 2009/12/28 07:32:15 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#ifndef _MPACK_DEBUG_H_
#define _MPACK_DEBUG_H_

#include <stdlib.h> //for abs
#include <stdio.h>
#define EPSILON  1e-13
#define TRUE     1
#define FALSE    0
#define GMP_P_FORMAT "%+21.16Fe"
#define P_FORMAT     "%+21.16e"

#if !defined _MPACK_DEBUG_C_
#define _MPACK_DEBUG_EXTERN_ extern
#else
#define _MPACK_DEBUG_EXTERN_
#endif

extern int blas_errno; //for BLAS xerbla dispatch
#include <iostream>
using std::cout;
using std::endl;
using std::complex;

void set_random_vector(REAL * vec, double *vec_d, int length);
void set_random_vector(COMPLEX * vec, complex < double >*vec_d, int length);
void set_random_vector(mpackint * vec, int *vec_d, int len, int n);
void set_random_vector(mpackint * vec, int *vec_d, int len, int n);
void set_random_number(REAL * alpha, double *alpha_d);
void set_random_number(COMPLEX * alpha, complex < double >*alpha_d);
void setrandpsdmat(int ldp, int n, REAL * p, double *pd);
void setrandpsdmat(int ldp, int n, COMPLEX * p, complex < double > *pd);
double l1norm(REAL* vec, double *vec_d, int len, int inc); 
double l1norm(COMPLEX* vec, complex < double >*vec_d, int len, int inc); 
double infnorm(REAL* vec, double *vec_d, int len, int inc); 
double infnorm(COMPLEX* vec, complex < double >*vec_d, int len, int inc); 
int infnorm(mpackint * vec, int *vec_d, int len, int inc);
double num_diff(REAL num1, double num2);
double num_diff(COMPLEX num1, complex <double> num2);
double rel_diff(REAL num1, double num2);
double rel_diff(COMPLEX num1, complex <double> num2);
void printmat(int N, int M, REAL * A, int LDA);
void printmat(int N, int M, double *A, int LDA);
void printmat(int N, int M, COMPLEX * A, int LDA);
void printmat(int N, int M, complex < double >*A, int LDA);

REAL mpf_randomnumber(void);
COMPLEX mpc_randomnumber(void);
int get_random_int_n(int n);

inline void printnum(double rtmp){
  printf(P_FORMAT, rtmp);
  return;
}
inline void printnum(complex<double> ctmp){
  printf(P_FORMAT P_FORMAT"i", ctmp.real(),ctmp.imag());
  return;
}

#if defined ___MPACK_BUILD_WITH_GMP___
_MPACK_DEBUG_EXTERN_ gmp_randclass *uniformrandom;
inline void printnum(mpf_class rtmp){
  gmp_printf(GMP_P_FORMAT, rtmp.get_mpf_t());
  return;
}
inline void printnum(mpc_class ctmp){
  gmp_printf(GMP_P_FORMAT GMP_P_FORMAT"i", ctmp.re.get_mpf_t(),ctmp.im.get_mpf_t());
  return;
}
#endif

#if defined ___MPACK_BUILD_WITH_QD___
inline void printnum(qd_real rtmp){
  cout.precision(16);
  if (rtmp>=0.0) {
    cout << "+" << rtmp;
  } else {
    cout << rtmp;
  }
  return;
}
inline void printnum(qd_complex rtmp){
  cout.precision(16);
  if (rtmp.re>=0.0) {
    cout << "+" << rtmp.re;
  } else {
    cout << rtmp.re;
  }
  if (rtmp.im>=0.0) {
    cout << "+" << rtmp.im << "i";
  } else {
    cout << rtmp.im << "i";
  }
  return;
}
#endif

#if defined ___MPACK_BUILD_WITH_DD___
inline void printnum(dd_real rtmp){
  cout.precision(16);
  if (rtmp>=0.0) {
    cout << "+" << rtmp;
  } else {
    cout << rtmp;
  }
  return;
}
inline void printnum(dd_complex rtmp){
  cout.precision(16);
  if (rtmp.re>=0.0) {
    cout << "+" << rtmp.re;
  } else {
    cout << rtmp.re;
  }
  if (rtmp.im>=0.0) {
    cout << "+" << rtmp.im << "i";
  } else {
    cout << rtmp.im << "i";
  }
  return;
}
#endif

inline int veclen (int n, int inc)
{
  return max(1,(1+(abs(n)-1)*abs(inc)));
} 

inline int matlen (int lda, int n)
{
//workaround for redefinition of std::max and max in mpack_config.h
  return max(1,abs(lda)*abs(n));
} 

#endif
