/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Cdotc.debug.cpp,v 1.5 2009/09/16 00:41:44 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#include <mblas.h>
#include <blas.h>
#include <mpack_debug.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MAX_INCX  10
#define MIN_INCX -10
#define MAX_INCY  10
#define MIN_INCY -10
#define MAX_N     10
#define MAX_ITER  10

void
Cdotc_test()
{
    int errorflag = FALSE;
    COMPLEX ctemp;
    complex < double >dtemp;

    for (int incx = MAX_INCX; incx >= MIN_INCX; incx--) {
	for (int incy = MAX_INCY; incy >= MIN_INCY; incy--) {
	    for (int n = 0; n < MAX_N; n++) {
                printf("# n:%d incx:%d, incy:%d\n", n, incx, incy);
		COMPLEX *x = new COMPLEX[veclen(n, incx)];
		COMPLEX *y = new COMPLEX[veclen(n, incy)];
		complex < double >*xd = new complex < double >[veclen(n, incx)];
		complex < double >*yd = new complex < double >[veclen(n, incy)];
		int j = 0;
		while (j < MAX_ITER) {
		    set_random_vector(x, xd, veclen(n, incx));
		    set_random_vector(y, yd, veclen(n, incy));
		    ctemp = Cdotc(n, x, incx, y, incy);
		    dtemp = zdotc_f77(&n, xd, &incx, yd, &incy);
		    double diff = num_diff(ctemp, dtemp);
		    if (fabs(diff) > EPSILON) {
			printf("error %.16e!!\n", diff);
			errorflag = TRUE;
		    }
		    j++;
		}
		delete[]x;
		delete[]y;
		delete[]xd;
		delete[]yd;
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Cdotc test failed...\n");
	exit(1);
    }
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Cdotc_test();
    printf("Cdotc test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
