/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Cgemm.debug.cpp,v 1.8 2009/09/16 00:41:44 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <blas.h>
#include <mpack_debug.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_M -2
#define MAX_M 4
#define MIN_N -2
#define MAX_N 4
#define MIN_K -2
#define MAX_K 4
#define MIN_LDA -2
#define MAX_LDA 4
#define MIN_LDB -2
#define MAX_LDB 4
#define MIN_LDC -2
#define MAX_LDC 4
#define MAX_ITER 3

void
Cgemm_test3(const char *transa, const char *transb, COMPLEX alpha,
    COMPLEX beta)
{
    int errorflag = FALSE;

    for (int k = MIN_K; k < MAX_K; k++) {
	for (int n = MIN_N; n < MAX_N; n++) {
	    for (int m = MIN_M; m < MAX_M; m++) {
		int minlda, minldb;
		if (Mlsame(transa, "N"))
		    minlda = max(1, m);
		else
		    minlda = max(1, k);
		if (Mlsame(transb, "N"))
		    minldb = max(1, k);
		else
		    minldb = max(1, n);

		for (int lda = minlda; lda < MAX_LDA; lda++) {
		    for (int ldb = minldb; ldb < MAX_LDB; ldb++) {
			for (int ldc = max(1, m); ldc < MAX_LDC; ldc++) {
#if defined VERBOSE_TEST
                          printf("#k is %d, n is %d, m is %d, lda is %d, ldb is %d, ldc is %d\n", k, n, m, lda, ldb, ldc);
                          printf("#transa is %s, transb is %s\n", transa,transb);
#endif
			    COMPLEX *A;
			    COMPLEX *B;
			    COMPLEX *C;
			    complex < double >*Ad;
			    complex < double >*Bd;
			    complex < double >*Cd;
			    int ka, kb;
			    if (Mlsame(transa, "N"))
				ka = k;
			    else
				ka = m;

			    if (Mlsame(transb, "N"))
				kb = n;
			    else
				kb = k;

			    A = new COMPLEX[matlen(lda, ka)];
			    B = new COMPLEX[matlen(ldb, kb)];
			    C = new COMPLEX[matlen(ldc, n)];
			    Ad = new complex < double >[matlen(lda, ka)];
			    Bd = new complex < double >[matlen(ldb, kb)];
			    Cd = new complex < double >[matlen(ldc, n)];

			    complex < double >alphad, betad;

			    alphad.real() = cast2double(alpha.re);
			    alphad.imag() = cast2double(alpha.im);
			    betad.real() = cast2double(beta.re);
			    betad.imag() = cast2double(beta.im);

			    for (int iter = 0; iter < MAX_ITER; iter++) {
				set_random_vector(A, Ad, matlen(lda, ka));
				set_random_vector(B, Bd, matlen(ldb, kb));
				set_random_vector(C, Cd, matlen(ldc, n));

 			        mpack_errno = 0; blas_errno = 0;
				Cgemm (transa, transb, m, n, k, alpha, A, lda,
				    B, ldb, beta, C, ldc);
				zgemm_f77(transa, transb, &m, &n, &k, &alphad,
				    Ad, &lda, Bd, &ldb, &betad, Cd, &ldc);
#if defined VERBOSE_TEST
				printf("errno: mpack %d, blas %d\n", mpack_errno, blas_errno);
#endif
				    if (mpack_errno != blas_errno) {
					printf("error in Mxerbla!!\n");
				        exit(1);
				    }

				double diff = infnorm(C, Cd, matlen(ldc, n), 1);
				if (fabs(diff) > EPSILON) {
				    printf("#error %.16e!!\n", diff);
				    errorflag = TRUE;
				} else {
				}
			    }
			    delete[]Cd;
			    delete[]Bd;
			    delete[]Ad;

			    delete[]C;
			    delete[]B;
			    delete[]A;
			}
		    }
		}
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Cgemm test failed...\n");
	exit(1);
    }
}

void
Cgemm_test2(const char *uplo, const char *trans)
{
    COMPLEX alpha;
    COMPLEX beta;
    COMPLEX Czero, Cone;

    Czero.re = 0.0;
    Czero.im = 0.0;
    Cone.re = 1.0;
    Cone.im = 0.0;

//a=0, b=*;
    alpha = Czero;
    beta = mpc_randomnumber();
    Cgemm_test3(uplo, trans, alpha, beta);

//a=*, b=0;
    alpha = mpc_randomnumber();
    beta = Czero;
    Cgemm_test3(uplo, trans, alpha, beta);

//a=*, b=1;
    alpha = mpc_randomnumber();
    beta = Cone;
    Cgemm_test3(uplo, trans, alpha, beta);

//alpha=*, beta=*
    alpha = mpc_randomnumber();
    beta = mpc_randomnumber();
    Cgemm_test3(uplo, trans, alpha, beta);

//a=0, b=0;
    alpha = Czero;
    beta = Czero;
    Cgemm_test3(uplo, trans, alpha, beta);

//a=0, b=1;
    alpha = Czero;
    beta = Cone;
    Cgemm_test3(uplo, trans, alpha, beta);

//a=1, b=0;
    alpha = Cone;
    beta = Czero;
    Cgemm_test3(uplo, trans, alpha, beta);

//a=1, b=1;
    alpha = Cone;
    beta = Cone;
    Cgemm_test3(uplo, trans, alpha, beta);

//a=1, b=*;
    alpha = Cone;
    beta = mpc_randomnumber();
    Cgemm_test3(uplo, trans, alpha, beta);
}

void
Cgemm_test()
{
    Cgemm_test2("N", "N");
    Cgemm_test2("N", "T");
    Cgemm_test2("N", "C");

    Cgemm_test2("T", "N");
    Cgemm_test2("T", "C");
    Cgemm_test2("T", "T");

    Cgemm_test2("C", "N");
    Cgemm_test2("C", "T");
    Cgemm_test2("C", "C");
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Cgemm_test();
    printf("Cgemm test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
