/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Cher2.debug.cpp,v 1.7 2009/09/16 00:41:44 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <blas.h>
#include <mpack_debug.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_INCX   -2
#define MAX_INCX   10
#define MIN_INCY   -2
#define MAX_INCY   10
#define MIN_N      -2
#define MAX_N      20
#define MAX_LDA    20
#define MAX_ITER   10

void
Cher2_test2(const char *uplo)
{
    int errorflag = FALSE;

    for (int n = MIN_N; n < MAX_N; n++) {
	for (int lda = max(n, 1); lda < MAX_LDA; lda++) {
	    for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
		for (int incy = MIN_INCY; incy < MAX_INCY; incy++) {
#if defined VERBOSE_TEST
                  printf("#n is %d, incx is %d, uplo is %s lda is %d.\n", n,
                      incx, uplo, lda);
#endif
		    COMPLEX alpha;
		    COMPLEX *x;
		    COMPLEX *y;
		    COMPLEX *A;

		    complex < double >alphad;
		    complex < double >*xd;
		    complex < double >*yd;
		    complex < double >*Ad;

		    x = new COMPLEX[veclen(n, incx)];
		    y = new COMPLEX[veclen(n, incy)];
		    A = new COMPLEX[matlen(lda, n)];
		    xd = new complex < double >[veclen(n, incx)];
		    yd = new complex < double >[veclen(n, incy)];
		    Ad = new complex < double >[matlen(lda, n)];

		    for (int i = 0; i < MAX_ITER; i++) {
			set_random_vector(A, Ad, matlen(lda, n));
			set_random_vector(x, xd, veclen(n, incx));
			set_random_vector(y, yd, veclen(n, incy));
			set_random_number(&alpha, &alphad);

			Cher2(uplo, n, alpha, x, incx, y, incy, A, lda);
			zher2_f77(uplo, &n, &alphad, xd, &incx, yd, &incy, Ad,
			    &lda);
#if defined VERBOSE_TEST
			    printf("errno: mpack %d, blas %d\n", mpack_errno, blas_errno);
#endif
			     if (mpack_errno != blas_errno) {
			       printf("error in Mxerbla!!\n");
			       exit(1);
			     }
		            mpack_errno = 0; blas_errno = 0;

		       double diff = infnorm(A, Ad, matlen(lda, n), 1);
			if (fabs(diff) > EPSILON) {
			    printf("#error %.16e!!\n", diff);
			    errorflag = TRUE;
			}
		    }
		    delete[]Ad;
		    delete[]xd;
		    delete[]A;
		    delete[]x;
		    delete[]y;
		    delete[]yd;
		}
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Cher2 test failed...\n");
	exit(1);
    }
}

void
Cher2_test()
{
    Cher2_test2("U");
    Cher2_test2("L");
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Cher2_test();
    printf("Cher2 test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}




