/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Cher2k.debug.cpp,v 1.7 2009/09/16 00:41:44 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <blas.h>
#include <mpack_debug.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_K -2
#define MAX_K 8
#define MIN_N -2
#define MAX_N 8
#define MIN_LDA -2
#define MAX_LDA 8
#define MIN_LDB -2
#define MAX_LDB 8
#define MIN_LDC -2
#define MAX_LDC 8
#define MAX_ITER 1

void
Cher2k_test3(const char *uplo, const char *trans, COMPLEX alpha,
    REAL beta)
{
    int errorflag = FALSE;
    for (int n = MIN_N; n < MAX_N; n++) {
	for (int k = MIN_K; k < MAX_K; k++) {
	    int minlda;
	    if (Mlsame(trans, "N"))
		minlda = max(1, n);
	    else
		minlda = max(1, k);
	    int minldb;
	    if (Mlsame(trans, "N"))
		minldb = max(1, n);
	    else
		minldb = max(1, k);
	    for (int lda = minlda; lda < MAX_LDA; lda++) {
		for (int ldb = minldb; ldb < MAX_LDB; ldb++) {
		    for (int ldc = max(1, n); ldc < MAX_LDC; ldc++) {
#if defined VERBOSE_TEST
			printf
			    ("#n is %d, k is %d, lda is %d, ldb is %d, ldc is %d ",
			    n, k, lda, ldb, ldc);
			printf("uplo is %s, trans is %s \n", uplo, trans);
#endif
			COMPLEX *A;
			COMPLEX *B;
			COMPLEX *C;
			complex < double >*Ad;
			complex < double >*Bd;
			complex < double >*Cd;
			int ka, kb;
			if (Mlsame(trans, "N"))
			    ka = k;
			else
			    ka = n;
			if (Mlsame(trans, "N"))
			    kb = k;
			else
			    kb = n;

			A = new COMPLEX[matlen(lda, ka)];
			B = new COMPLEX[matlen(ldb, kb)];
			C = new COMPLEX[matlen(ldc, n)];
			Ad = new complex < double >[matlen(lda, ka)];
			Bd = new complex < double >[matlen(ldb, kb)];
			Cd = new complex < double >[matlen(ldc, n)];

			complex < double >alphad;
			double betad;

			alphad.real() = cast2double(alpha.re);
			alphad.imag() = cast2double(alpha.im);
			betad = cast2double(beta);

			for (int iter = 0; iter < MAX_ITER; iter++) {
			    set_random_vector(A, Ad, matlen(lda, ka));
			    set_random_vector(B, Bd, matlen(ldb, kb));
			    set_random_vector(C, Cd, matlen(ldc, n));

		            mpack_errno = 0; blas_errno = 0;
			    Cher2k(uplo, trans, n, k, alpha, A, lda, B, ldb,
				beta, C, ldc);
			    zher2k_f77(uplo, trans, &n, &k, &alphad, Ad, &lda,
				Bd, &ldb, &betad, Cd, &ldc);
#if defined VERBOSE_TEST
			    printf("errno: mpack %d, blas %d\n", mpack_errno, blas_errno);
#endif
			     if (mpack_errno != blas_errno) {
			       printf("error in Mxerbla!!\n");
			       exit(1);
			     }
		            mpack_errno = 0; blas_errno = 0;

			    double diff = infnorm(C, Cd, matlen(ldc, n), 1);
			    if (fabs(diff) > EPSILON) {
				printf("#error %.16e!!\n", diff);
				errorflag = TRUE;
			    }
			}
			delete[]Cd;
			delete[]Bd;
			delete[]Ad;

			delete[]C;
			delete[]B;
			delete[]A;
		    }
		}
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Cher2k test failed...\n");
	exit(1);
    }
}

void
Cher2k_test2(const char *uplo, const char *trans)
{
    COMPLEX alpha;
    REAL beta;
    COMPLEX Czero, Cone;
    REAL Rzero, Rone;

    Rone = 1.0;
    Rzero = 0.0;
    Czero.re = 0.0;
    Czero.im = 0.0;
    Cone.re = 1.0;
    Cone.im = 0.0;

//a=0, b=*;
    alpha = Czero;
    beta = mpf_randomnumber();
    Cher2k_test3(uplo, trans, alpha, beta);

//a=*, b=0;
    alpha = mpc_randomnumber();
    beta = Rzero;
    Cher2k_test3(uplo, trans, alpha, beta);

//a=*, b=1;
    alpha = mpc_randomnumber();
    beta = Rone;
    Cher2k_test3(uplo, trans, alpha, beta);

//alpha=*, beta=*
    alpha = mpc_randomnumber();
    beta = mpf_randomnumber();
    Cher2k_test3(uplo, trans, alpha, beta);

//a=0, b=0;
    alpha = Czero;
    beta = Rzero;
    Cher2k_test3(uplo, trans, alpha, beta);

//a=0, b=1;
    alpha = Czero;
    beta = Rone;
    Cher2k_test3(uplo, trans, alpha, beta);

//a=1, b=0;
    alpha = Cone;
    beta = Rzero;
    Cher2k_test3(uplo, trans, alpha, beta);

//a=1, b=1;
    alpha = Cone;
    beta = Rone;
    Cher2k_test3(uplo, trans, alpha, beta);

//a=1, b=*;
    alpha = Cone;
    beta = mpf_randomnumber();
    Cher2k_test3(uplo, trans, alpha, beta);
}

void
Cher2k_test()
{
    Cher2k_test2("U", "N");
    Cher2k_test2("U", "C");
    Cher2k_test2("L", "N");
    Cher2k_test2("L", "C");
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Cher2k_test();
    printf("Cher2k test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
