/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Ctrmm.debug.cpp,v 1.7 2009/09/16 00:41:44 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <blas.h>
#include <mpack_debug.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_N -2
#define MAX_N 8
#define MIN_M -2
#define MAX_M 8
#define MIN_LDA -2
#define MAX_LDA 8
#define MIN_LDB -2
#define MAX_LDB 8
#define MAX_ITER 3

void
Ctrmm_test3(const char *side, const char *uplo, const char *transa,
    const char *diag, COMPLEX alpha)
{
    int errorflag = FALSE;

    for (int m = MIN_M; m < MAX_M; m++) {
	for (int n = MIN_N; n < MAX_N; n++) {
	    int minlda;
	    if (Mlsame(side, "L"))
		minlda = max(1, m);
	    else
		minlda = max(1, n);
	    for (int lda = minlda; lda < MAX_LDA; lda++) {
		for (int ldb = max(1, m); ldb < MAX_LDB; ldb++) {
		    int k;
		    if (Mlsame(side, "L"))
			k = m;
		    if (Mlsame(side, "R"))
			k = n;
#if defined VERBOSE_TEST
		    printf("#n is %d, m is %d, lda is %d, ldb is %d\n", n, m,
			lda, ldb);
		    printf
			("#side is %s, uplo is %s transa is %s diag is %s \n",
			side, uplo, transa, diag);
#endif
		    COMPLEX *A;
		    COMPLEX *B;
		    complex < double >*Ad;
		    complex < double >*Bd;

		    A = new COMPLEX[matlen(lda, k)];
		    B = new COMPLEX[matlen(ldb, n)];
		    Ad = new complex < double >[matlen(lda, k)];
		    Bd = new complex < double >[matlen(ldb, n)];

		    complex < double >alphad;

		    alphad.real() = cast2double(alpha.re);
		    alphad.imag() = cast2double(alpha.im);

		    for (int iter = 0; iter < MAX_ITER; iter++) {
			set_random_vector(A, Ad, matlen(lda, k));
			set_random_vector(B, Bd, matlen(ldb, n));

		        mpack_errno = 0; blas_errno = 0;
			Ctrmm(side, uplo, transa, diag, m, n, alpha, A, lda, B,
			    ldb);
			ztrmm_f77(side, uplo, transa, diag, &m, &n, &alphad,
			    Ad, &lda, Bd, &ldb);
#if defined VERBOSE_TEST
			printf("errno: mpack %d, blas %d\n", mpack_errno, blas_errno);
#endif
			if (mpack_errno != blas_errno) {
			      printf("error in Mxerbla!!\n");
			     exit(1);
			}
			
			double diff = infnorm(B, Bd, matlen(ldb, n), 1);
			if (fabs(diff) > EPSILON) {
			    printf("#error %.16e!!\n", diff);
			    errorflag = TRUE;
			}
		    }
		    delete[]Bd;
		    delete[]Ad;
		    delete[]B;
		    delete[]A;
		}
	    }
	}
    }

    if (errorflag == TRUE) {
	printf("Ctrmm test failed...\n");
	exit(1);
    }
}

void
Ctrmm_test2(const char *side, const char *uplo, const char *transa,
    const char *diag)
{
    COMPLEX alpha;
    COMPLEX Cone, Czero;

    Czero.re = 0.0;
    Czero.im = 0.0;
    Cone.re = 1.0;
    Cone.im = 0.0;

//a=*
    alpha = mpc_randomnumber();
    Ctrmm_test3(side, uplo, transa, diag, alpha);

//a=1
    alpha = Cone;
    Ctrmm_test3(side, uplo, transa, diag, alpha);

//a=0
    alpha = Czero;
    Ctrmm_test3(side, uplo, transa, diag, alpha);
}

void
Ctrmm_test()
{
    Ctrmm_test2("L", "U", "N", "U");
    Ctrmm_test2("L", "U", "N", "N");
    Ctrmm_test2("L", "U", "T", "U");
    Ctrmm_test2("L", "U", "T", "N");
    Ctrmm_test2("L", "U", "C", "U");
    Ctrmm_test2("L", "U", "C", "N");
    Ctrmm_test2("L", "L", "N", "U");
    Ctrmm_test2("L", "L", "N", "N");
    Ctrmm_test2("L", "L", "T", "U");
    Ctrmm_test2("L", "L", "T", "N");
    Ctrmm_test2("L", "L", "C", "U");
    Ctrmm_test2("L", "L", "C", "N");

    Ctrmm_test2("R", "U", "N", "U");
    Ctrmm_test2("R", "U", "N", "N");
    Ctrmm_test2("R", "U", "T", "U");
    Ctrmm_test2("R", "U", "T", "N");
    Ctrmm_test2("R", "U", "C", "U");
    Ctrmm_test2("R", "U", "C", "N");
    Ctrmm_test2("R", "L", "N", "U");
    Ctrmm_test2("R", "L", "N", "N");
    Ctrmm_test2("R", "L", "T", "U");
    Ctrmm_test2("R", "L", "T", "N");
    Ctrmm_test2("R", "L", "C", "U");
    Ctrmm_test2("R", "L", "C", "N");
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Ctrmm_test();
    printf("Ctrmm test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
