/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Rgbmv.debug.cpp,v 1.5 2009/09/15 12:49:54 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <blas.h>
#include <mpack_debug.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_M -2
#define MAX_M 10
#define MIN_N -2
#define MAX_N 10
#define MIN_KL -2
#define MAX_KL 10
#define MIN_KU -2
#define MAX_KU 10
#define MIN_LDA -2
#define MAX_LDA 10
#define MIN_INCX -3
#define MAX_INCX 3
#define MIN_INCY -3
#define MAX_INCY 3
#define MAX_ITER 1

void
Rgbmv_test3(const char *trans, REAL alpha, REAL beta)
{
    int errorflag = FALSE;

    for (int incy = MIN_INCY; incy < MAX_INCY; incy++) {
	for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
	    for (int lda = MIN_LDA; lda < MAX_LDA; lda++) {
		for (int ku = MIN_KU; ku < MAX_KU; ku++) {
		    for (int kl = MIN_KL; kl < MAX_KL; kl++) {
			for (int n = MIN_N; n < MAX_N; n++) {
			    for (int m = MIN_M; m < MAX_M; m++) {
#if defined VERBOSE_TEST
				printf
				    ("#m is %d, n is %d, kl is %d, ku is %d, lda is %d, incx is %d, incy is %d, trans is %s.\n",
				    m, n, kl, ku, lda, incx, incy, trans);
#endif
				REAL *x;
				REAL *y;
				REAL *A;
				double *xd;
				double *yd;
				double *Ad;

				if (Mlsame(trans, "N")) {
				    x = new REAL[veclen(n, incx)];
				    y = new REAL[veclen(m, incy)];
				    A = new REAL[matlen(lda, n)];
				    xd = new double[veclen(n, incx)];
				    yd = new double[veclen(m, incy)];
				    Ad = new double[matlen(lda, n)];
				} else {
				    x = new REAL[veclen(m, incx)];
				    y = new REAL[veclen(n, incy)];
				    A = new REAL[matlen(lda, n)];
				    xd = new double[veclen(m, incx)];
				    yd = new double[veclen(n, incy)];
				    Ad = new double[matlen(lda, n)];
				}

				double alphad, betad;

				alphad = cast2double(alpha);
				betad = cast2double(beta);

				for (int iter = 0; iter < MAX_ITER; iter++) {
				    if (Mlsame(trans, "N")) {
					set_random_vector(x, xd, veclen(n,
						incx));
					set_random_vector(y, yd, veclen(m,
						incy));
					set_random_vector(A, Ad, matlen(lda,
						n));
				    } else {
					set_random_vector(x, xd, veclen(m,
						incx));
					set_random_vector(y, yd, veclen(n,
						incy));
					set_random_vector(A, Ad, matlen(lda,
						n));
				    }
				    mpack_errno = 0; blas_errno = 0;
				    Rgbmv(trans, m, n, kl, ku, alpha, A, lda,
					x, incx, beta, y, incy);
				    dgbmv_f77(trans, &m, &n, &kl, &ku, &alphad,
					Ad, &lda, xd, &incx, &betad, yd,
					&incy);
                                    #if defined VERBOSE_TEST
				    printf("errno: mpack %d, blas %d\n", mpack_errno, blas_errno);
                                    #endif	
				    if (mpack_errno != blas_errno) {
				      printf("error in Mxerbla!!\n");
				      exit(1);
				    }
				    double diff;
				    if (Mlsame(trans, "N")) {
					diff =
					    infnorm(y, yd, veclen(m, incy), 1);
				    } else {
					diff =
					    infnorm(y, yd, veclen(n, incy), 1);
				    }
				    if (fabs(diff) > EPSILON) {
					printf("error %.16e!\n", diff);
					errorflag = TRUE;
				    } else {
//                                      printf("okay\n");
				    }
				}
				delete[]x;
				delete[]y;
				delete[]A;
				delete[]xd;
				delete[]yd;
				delete[]Ad;
			    }
			}
		    }
		}
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Rgbmv test failed...\n");
	exit(1);
    }

}

void
Rgbmv_test2(const char *trans)
{
    REAL alpha;
    REAL beta;
    REAL Rzero, Rone;

    Rzero = 0.0;
    Rone = 1.0;

//a=0, b=*;
    alpha = Rzero;
    beta = mpf_randomnumber();
    Rgbmv_test3(trans, alpha, beta);

//a=*, b=0;
    alpha = mpf_randomnumber();
    beta = Rzero;
    Rgbmv_test3(trans, alpha, beta);

//a=*, b=1;
    alpha = mpf_randomnumber();
    beta = Rone;
    Rgbmv_test3(trans, alpha, beta);

//alpha=*, beta=*
    alpha = mpf_randomnumber();
    beta = mpf_randomnumber();
    Rgbmv_test3(trans, alpha, beta);

//a=0, b=0;
    alpha = Rzero;
    beta = Rzero;
    Rgbmv_test3(trans, alpha, beta);

//a=0, b=1;
    alpha = Rzero;
    beta = Rone;
    Rgbmv_test3(trans, alpha, beta);

//a=1, b=0;
    alpha = Rone;
    beta = Rzero;
    Rgbmv_test3(trans, alpha, beta);

//a=1, b=1;
    alpha = Rone;
    beta = Rone;
    Rgbmv_test3(trans, alpha, beta);

//a=1, b=*;
    alpha = Rone;
    beta = mpf_randomnumber();
    Rgbmv_test3(trans, alpha, beta);

}

void
Rgbmv_test()
{
    Rgbmv_test2("N");
    Rgbmv_test2("T");
    Rgbmv_test2("C");
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Rgbmv_test();
    printf("Rgbmv test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
