/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Rrotg.debug.cpp,v 1.5 2009/09/16 00:55:18 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <blas.h>
#include <mpack_debug.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MAX_ITER  100

void
Rrotg_test()
{
    int errorflag = FALSE;

    REAL a;
    REAL b;
    REAL c;
    REAL s;

    double ad;
    double bd;
    double cd;
    double sd;
    int j = 0;

    while (j < MAX_ITER) {
	set_random_number(&a, &ad);
	set_random_number(&b, &bd);
	set_random_number(&c, &cd);

	Rrotg(&a, &b, &c, &s);
	drotg_f77(&ad, &bd, &cd, &sd);

	double diff[4];

	diff[0] = cast2double(a) - ad;
	diff[1] = cast2double(b) - bd;
	diff[2] = cast2double(c) - cd;
	diff[3] = cast2double(s) - sd;
#if defined VERBOSE_TEST
	for (int p = 0; p < 4; p++) {
	    printf("diff[%d]=%.16e\n", p, diff[p]);
	}
#endif
	for (int p = 0; p < 4; p++) {
	    if (fabs(diff[p]) > EPSILON) {
		printf("error %d, %.16e!!\n", p, diff[p]);
		errorflag = TRUE;
	    }
	}
	j++;
    }
    if (errorflag == TRUE) {
	printf("Rrotg test failed...\n");
	exit(1);
    }

}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Rrotg_test();
    printf("Rrotg test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}

