/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: complex.debug.cpp,v 1.9 2009/09/24 05:39:01 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <complex>
#include <blas.h>
#include <mblas.h>
#include <mpack_debug.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MAX_ITER 10

void
mpc_subst_test1()
{
    printf("Complex <= Real test \n");
    COMPLEX Ctemp1;
    COMPLEX Ctemp2;
    REAL Ftemp;
    int flag = 0;
    for (int i = 0; i < MAX_ITER; i++) {
      	Ctemp1 = mpc_randomnumber();
#if defined VERBOSE_TEST
        cout << "c1       " ; printnum(Ctemp1);cout << endl;
#endif
	Ftemp = mpf_randomnumber();
	Ctemp1 = Ftemp;		//this is what we want to test
#if defined VERBOSE_TEST
        cout << "r1=      " ; printnum(Ftemp); cout << endl;
        cout << "c2=      " ; printnum(Ctemp1);cout << endl;
        cout << "residue=r1-c2 " << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
#endif
	Ctemp2 = Ctemp1 - Ftemp;
	REAL diff = abs(Ctemp2);
#if defined VERBOSE_TEST
	cout << "DIFF "; printnum(diff); cout << endl;
#endif
	if (abs(diff) > EPSILON) {
	    flag = 1;
	}
    }
    if (flag) {
	printf("Complex <= Real subst test failed\n");
	exit(1);
    } else {
	printf("Complex <= Real subst test passed\n");
    }
}

void
mpc_abs_test()
{
    printf("Complex abs test \n");
    COMPLEX Ctemp1;
    REAL Ftemp1;
    REAL diff;
    double ftemp1;
    int flag = 0;

    for (int i = 0; i < MAX_ITER; i++) {
	Ctemp1 = mpc_randomnumber();
	complex < double >ctemp1(cast2double(Ctemp1.re), cast2double(Ctemp1.im));

	Ftemp1 = abs(Ctemp1);
	ftemp1 = abs(ctemp1);
	diff = abs(Ftemp1 - ftemp1);
#if defined VERBOSE_TEST
        cout << "C1 = " ; printnum(Ctemp1);cout << endl;
        cout << "F1 = " ; printnum(Ftemp1);cout << endl;
        cout << endl;
        cout << "c1 = " ; printnum(ctemp1);cout << endl;
        cout << "f1 = " ; printnum(ftemp1);cout << endl;
        cout << "residue=sqrt(c1*conj(c1))-f1 " << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
        cout << "residue=F1-f1 " << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
	cout << "DIFF = "; printnum(diff); cout << endl;  cout << endl;
#endif
	if (abs(Ftemp1 - ftemp1) > EPSILON) {
	    flag = 1;
	}
    }
    if (flag) {
	printf("Complex ABS test failed\n");
	exit(1);
    } else {
	printf("Complex ABS test passed\n");
    }
}

void
mpc_add_test1()
{
    printf("Complex + Complex test \n");
    COMPLEX Ctemp1;
    COMPLEX Ctemp2;
    COMPLEX Ctemp3;
    REAL Ftemp;
    REAL diff;

    complex < double >ctemp1;
    complex < double >ctemp2;
    complex < double >ctemp3;

    int flag = 0;

    for (int i = 0; i < MAX_ITER; i++) {
	Ctemp1 = mpc_randomnumber();
	Ctemp2 = mpc_randomnumber();

	ctemp1.real() = cast2double(Ctemp1.re);
	ctemp1.imag() = cast2double(Ctemp1.im);
	ctemp2.real() = cast2double(Ctemp2.re);
	ctemp2.imag() = cast2double(Ctemp2.im);

	Ctemp3 = Ctemp1 + Ctemp2;
	ctemp3 = ctemp1 + ctemp2;
#if defined VERBOSE_TEST
        cout << "C1 = " ; printnum(Ctemp1);cout << endl;
        cout << "C2 = " ; printnum(Ctemp2);cout << endl;
        cout << "C3 = " ; printnum(Ctemp3);cout << endl;
        cout << endl;
        cout << "c1 = " ; printnum(ctemp1);cout << endl;
        cout << "c2 = " ; printnum(ctemp2);cout << endl;
        cout << "c3 = " ; printnum(ctemp3);cout << endl;
#endif
	diff = abs(Ctemp3 - ctemp3);

#if defined VERBOSE_TEST
        cout << "residue=C1+C2-C3" << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
        cout << "residue=C3-c3 " << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
	cout << "DIFF = "; printnum(diff); cout << endl;  cout << endl;
#endif
	if (abs(diff) > EPSILON) {
	    flag = 1;
	}
    }
    if (flag) {
	printf("Complex + Complex ADD test failed\n");
	exit(1);
    } else {
	printf("Complex + Complex ADD test passed\n");
    }
}

void
mpc_add_test2()
{
    printf("Complex + Real test \n");
    COMPLEX Ctemp1;
    REAL Ftemp2;
    COMPLEX Ctemp3;

    REAL Ftemp;
    REAL diff;

    complex < double >ctemp1;
    double ftemp2;
    complex < double >ctemp3;

    int flag = 0;

    for (int i = 0; i < MAX_ITER; i++) {
	Ctemp1 = mpc_randomnumber();
	Ftemp2 = mpf_randomnumber();

	ctemp1.real() = cast2double(Ctemp1.re);
	ctemp1.imag() = cast2double(Ctemp1.im);
	ftemp2 = cast2double(Ftemp2);
	Ctemp3 = Ctemp1 + Ftemp2;
	ctemp3 = ctemp1 + ftemp2;
#if defined VERBOSE_TEST
        cout << "C1 = " ; printnum(Ctemp1);cout << endl;
        cout << "F2 = " ; printnum(Ftemp2);cout << endl;
        cout << "C3 = " ; printnum(Ctemp3);cout << endl;
        cout << endl;
        cout << "c1 = " ; printnum(ctemp1);cout << endl;
        cout << "f2 = " ; printnum(ftemp2);cout << endl;
        cout << "c3 = " ; printnum(ctemp3);cout << endl;
#endif
	diff = abs(Ctemp3 - ctemp3);
#if defined VERBOSE_TEST
        cout << "residue=C1+F2-C3" << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
        cout << "residue=C3-c3 " << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
	cout << "DIFF = "; printnum(diff); cout << endl;  cout << endl;
#endif
	if (abs(diff) > EPSILON) {
	    flag = 1;
	}
    }
    if (flag) {
	printf("Complex + Real ADD test failed\n");
	exit(1);
    } else {
	printf("Complex + Real ADD test passed\n");
    }
}

void
mpc_add_test3()
{
    printf("Real + Complex test \n");
    COMPLEX Ctemp2;
    REAL Ftemp1;
    COMPLEX Ctemp3;

    REAL Ftemp;
    REAL diff;

    complex < double >ctemp2;
    double ftemp1;
    complex < double >ctemp3;

    int flag = 0;

    for (int i = 0; i < MAX_ITER; i++) {
	Ftemp1 = mpf_randomnumber();
	Ctemp2 = mpc_randomnumber();

	ctemp2.real() = cast2double(Ctemp2.re);
	ctemp2.imag() = cast2double(Ctemp2.im);
	ftemp1 = cast2double(Ftemp1);

	Ctemp3 = Ftemp1 + Ctemp2;
	ctemp3 = ftemp1 + ctemp2;
#if defined VERBOSE_TEST
        cout << "F1 = " ; printnum(Ftemp1);cout << endl;
        cout << "C2 = " ; printnum(Ctemp2);cout << endl;
        cout << "C3 = " ; printnum(Ctemp3);cout << endl;
        cout << endl;
        cout << "f1 = " ; printnum(ftemp1);cout << endl;
        cout << "c2 = " ; printnum(ctemp2);cout << endl;
        cout << "c3 = " ; printnum(ctemp3);cout << endl;
#endif
	diff = abs(Ctemp3 - ctemp3);
#if defined VERBOSE_TEST
        cout << "residue=F1+C2-C3" << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
        cout << "residue=C3-c3 " << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
	cout << "DIFF = "; printnum(diff); cout << endl;  cout << endl;
#endif
	if (abs(diff) > EPSILON) {
	    flag = 1;
	}
    }
    if (flag) {
	printf("Real + Complex test failed\n");
	exit(1);
    } else {
	printf("Real + Complex test passed\n");
    }
}

void
mpc_mul_test1()
{
    printf("Complex * Complex test \n");
    COMPLEX Ctemp1;
    COMPLEX Ctemp2;
    COMPLEX Ctemp3;
    REAL Ftemp;
    REAL diff;

    complex < double >ctemp1;
    complex < double >ctemp2;
    complex < double >ctemp3;

    int flag = 0;

    for (int i = 0; i < MAX_ITER; i++) {
	Ctemp1 = mpc_randomnumber();
	Ctemp2 = mpc_randomnumber();

	ctemp1.real() = cast2double(Ctemp1.re);
	ctemp1.imag() = cast2double(Ctemp1.im);
	ctemp2.real() = cast2double(Ctemp2.re);
	ctemp2.imag() = cast2double(Ctemp2.im);
	Ctemp3 = Ctemp1 * Ctemp2;
	ctemp3 = ctemp1 * ctemp2;

#if defined VERBOSE_TEST
        cout << "C1 = " ; printnum(Ctemp1);cout << endl;
        cout << "C2 = " ; printnum(Ctemp2);cout << endl;
        cout << "C3 = " ; printnum(Ctemp3);cout << endl;
        cout << endl;
        cout << "c1 = " ; printnum(ctemp1);cout << endl;
        cout << "c2 = " ; printnum(ctemp2);cout << endl;
        cout << "c3 = " ; printnum(ctemp3);cout << endl;
#endif
	diff = abs(Ctemp3 - ctemp3);
#if defined VERBOSE_TEST
        cout << "residue=C1*C2-C3" << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
        cout << "residue=C3-c3 " << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
	cout << "DIFF = "; printnum(diff); cout << endl;  cout << endl;
#endif
	if (abs(diff) > EPSILON) {
	    flag = 1;
	}
    }
    if (flag) {
	printf("Complex * Complex test failed\n");
	exit(1);
    } else {
	printf("Complex * Complex test passed\n");
    }
}

void
mpc_mul_test2()
{
    printf("Complex * Real test \n");
    COMPLEX Ctemp1;
    REAL Ftemp2;
    COMPLEX Ctemp3;
    REAL Ftemp;
    REAL diff;

    complex < double >ctemp1;
    double ftemp2;
    complex < double >ctemp3;

    int flag = 0;

    for (int i = 0; i < MAX_ITER; i++) {
	Ctemp1 = mpc_randomnumber();
	Ftemp2 = mpf_randomnumber();

	ctemp1.real() = cast2double(Ctemp1.re);
	ctemp1.imag() = cast2double(Ctemp1.im);
	ftemp2 = cast2double(Ftemp2);
	Ctemp3 = Ctemp1 * Ftemp2;
	ctemp3 = ctemp1 * ftemp2;

#if defined VERBOSE_TEST
        cout << "C1 = " ; printnum(Ctemp1);cout << endl;
        cout << "F2 = " ; printnum(Ftemp2);cout << endl;
        cout << "C3 = " ; printnum(Ctemp3);cout << endl;
        cout << endl;
        cout << "c1 = " ; printnum(ctemp1);cout << endl;
        cout << "f2 = " ; printnum(ftemp2);cout << endl;
        cout << "c3 = " ; printnum(ctemp3);cout << endl;
#endif
	diff = abs(Ctemp3 - ctemp3);
#if defined VERBOSE_TEST
        cout << "residue=C1*F2-C3" << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
        cout << "residue=C3-c3 " << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
	cout << "DIFF = "; printnum(diff); cout << endl;  cout << endl;
#endif
	if (abs(diff) > EPSILON) {
	    flag = 1;
	}
    }
    if (flag) {
	printf("Complex * Real test failed\n");
	exit(1);
    } else {
	printf("Complex * Real test passed\n");
    }
}

void
mpc_mul_test3()
{
    printf("Real * Complex test \n");
    COMPLEX Ctemp2;
    REAL Ftemp1;
    COMPLEX Ctemp3;
    REAL Ftemp;
    REAL diff;

    complex < double >ctemp2;
    double ftemp1;
    complex < double >ctemp3;

    int flag = 0;

    for (int i = 0; i < MAX_ITER; i++) {
	Ftemp1 = mpf_randomnumber();
	Ctemp2 = mpc_randomnumber();

	ctemp2.real() = cast2double(Ctemp2.re);
	ctemp2.imag() = cast2double(Ctemp2.im);
	ftemp1 = cast2double(Ftemp1);
	Ctemp3 = Ftemp1 * Ctemp2;
	ctemp3 = ftemp1 * ctemp2;
#if defined VERBOSE_TEST
        cout << "F1 = " ; printnum(Ftemp1);cout << endl;
        cout << "C2 = " ; printnum(Ctemp2);cout << endl;
        cout << "C3 = " ; printnum(Ctemp3);cout << endl;
        cout << endl;
        cout << "f1 = " ; printnum(ftemp1);cout << endl;
        cout << "c2 = " ; printnum(ctemp2);cout << endl;
        cout << "c3 = " ; printnum(ctemp3);cout << endl;
#endif
	diff = abs(Ctemp3 - ctemp3);
#if defined VERBOSE_TEST
        cout << "residue=C1*F2-C3" << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
        cout << "residue=C3-c3 " << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
	cout << "DIFF = "; printnum(diff); cout << endl;  cout << endl;
#endif
	if (abs(diff) > EPSILON) {
	    flag = 1;
	}
    }
    if (flag) {
	printf("Complex * Real test failed\n");
	exit(1);
    } else {
	printf("Complex * Real test passed\n");
    }
}

void
mpc_div_test1()
{
    printf("Complex / Complex test \n");
    COMPLEX Ctemp1;
    COMPLEX Ctemp2;
    COMPLEX Ctemp3;
    REAL Ftemp;
    REAL diff;

    complex < double >ctemp1;
    complex < double >ctemp2;
    complex < double >ctemp3;

    int flag = 0;

    for (int i = 0; i < MAX_ITER; i++) {
	Ctemp1 = mpc_randomnumber();
	Ctemp2 = mpc_randomnumber();

	ctemp1.real() = cast2double(Ctemp1.re);
	ctemp1.imag() = cast2double(Ctemp1.im);
	ctemp2.real() = cast2double(Ctemp2.re);
	ctemp2.imag() = cast2double(Ctemp2.im);
	Ctemp3 = Ctemp1 / Ctemp2;
	ctemp3 = ctemp1 / ctemp2;
#if defined VERBOSE_TEST
        cout << "C1 = " ; printnum(Ctemp1);cout << endl;
        cout << "C2 = " ; printnum(Ctemp2);cout << endl;
        cout << "C3 = " ; printnum(Ctemp3);cout << endl;
        cout << endl;
        cout << "c1 = " ; printnum(ctemp1);cout << endl;
        cout << "c2 = " ; printnum(ctemp2);cout << endl;
        cout << "c3 = " ; printnum(ctemp3);cout << endl;
#endif
	diff = abs(Ctemp3 - ctemp3);
#if defined VERBOSE_TEST
        cout << "residue=C1/F2-C3" << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
        cout << "residue=C3-c3 " << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
	cout << "DIFF = "; printnum(diff); cout << endl;  cout << endl;
#endif
	if (abs(diff) > EPSILON) {
	    flag = 1;
	}
    }
    if (flag) {
	printf("Complex / Complex test failed\n");
	exit(1);
    } else {
	printf("Complex / Complex test passed\n");
    }
}

void
mpc_div_test2()
{
    printf("Complex / Real test \n");
    COMPLEX Ctemp1;
    REAL Ftemp2;
    COMPLEX Ctemp3;
    REAL Ftemp;
    REAL diff;

    complex < double >ctemp1;
    double ftemp2;
    complex < double >ctemp3;

    int flag = 0;

    for (int i = 0; i < MAX_ITER; i++) {
	Ctemp1 = mpc_randomnumber();
	Ftemp2 = mpf_randomnumber();

	ctemp1.real() = cast2double(Ctemp1.re);
	ctemp1.imag() = cast2double(Ctemp1.im);
	ftemp2 = cast2double(Ftemp2);
	Ctemp3 = Ctemp1 / Ftemp2;
	ctemp3 = ctemp1 / ftemp2;
#if defined VERBOSE_TEST
        cout << "C1 = " ; printnum(Ctemp1);cout << endl;
        cout << "f2 = " ; printnum(Ftemp2);cout << endl;
        cout << "C3 = " ; printnum(Ctemp3);cout << endl;
        cout << endl;
        cout << "c1 = " ; printnum(ctemp1);cout << endl;
        cout << "f2 = " ; printnum(ftemp2);cout << endl;
        cout << "c3 = " ; printnum(ctemp3);cout << endl;
#endif
	diff = abs(Ctemp3 - ctemp3);
#if defined VERBOSE_TEST
        cout << "residue=C1/F2-C3" << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
        cout << "residue=C3-c3 " << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
	cout << "DIFF = "; printnum(diff); cout << endl;  cout << endl;
#endif
	if (abs(diff) > EPSILON) {
	    flag = 1;
	}
    }
    if (flag) {
	printf("Complex / Real test failed\n");
    } else {
	printf("Complex / Real test passed\n");
    }
}

void
mpc_div_test3()
{
    printf("Real / Complex test \n");
    COMPLEX Ctemp2;
    REAL Ftemp1;
    COMPLEX Ctemp3;
    REAL Ftemp;
    REAL diff;

    complex < double >ctemp2;
    double ftemp1;
    complex < double >ctemp3;

    int flag = 0;

    for (int i = 0; i < MAX_ITER; i++) {
	Ftemp1 = mpf_randomnumber();
	Ctemp2 = mpc_randomnumber();

	ctemp2.real() = cast2double(Ctemp2.re);
	ctemp2.imag() = cast2double(Ctemp2.im);
	ftemp1 = cast2double(Ftemp1);
	Ctemp3 = Ftemp1 / Ctemp2;
	ctemp3 = ftemp1 / ctemp2;
#if defined VERBOSE_TEST
        cout << "F1 = " ; printnum(Ftemp1);cout << endl;
        cout << "C2 = " ; printnum(Ctemp2);cout << endl;
        cout << "C3 = " ; printnum(Ctemp3);cout << endl;
        cout << endl;
        cout << "f1 = " ; printnum(ftemp1);cout << endl;
        cout << "c2 = " ; printnum(ctemp2);cout << endl;
        cout << "c3 = " ; printnum(ctemp3);cout << endl;
#endif
	diff = abs(Ctemp3 - ctemp3);
#if defined VERBOSE_TEST
        cout << "residue=F1/C2-C3" << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
        cout << "residue=C3-c3 " << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
	cout << "DIFF = "; printnum(diff); cout << endl;  cout << endl;
#endif
	if (abs(diff) > EPSILON) {
	    flag = 1;
	}
    }
    if (flag) {
	printf("Real / Complex test failed\n");
	exit(1);
    } else {
	printf("Real / Complex test passed\n");
    }
}

void
mpc_sub_test1()
{
    printf("Complex - Complex test \n");
    COMPLEX Ctemp1;
    COMPLEX Ctemp2;
    COMPLEX Ctemp3;
    REAL Ftemp;
    REAL diff;

    complex < double >ctemp1;
    complex < double >ctemp2;
    complex < double >ctemp3;

    int flag = 0;

    for (int i = 0; i < MAX_ITER; i++) {
	Ctemp1 = mpc_randomnumber();
	Ctemp2 = mpc_randomnumber();

	ctemp1.real() = cast2double(Ctemp1.re);
	ctemp1.imag() = cast2double(Ctemp1.im);
	ctemp2.real() = cast2double(Ctemp2.re);
	ctemp2.imag() = cast2double(Ctemp2.im);
	Ctemp3 = Ctemp1 - Ctemp2;
	ctemp3 = ctemp1 - ctemp2;
#if defined VERBOSE_TEST
        cout << "C1 = " ; printnum(Ctemp1);cout << endl;
        cout << "C2 = " ; printnum(Ctemp2);cout << endl;
        cout << "C3 = " ; printnum(Ctemp3);cout << endl;
        cout << endl;
        cout << "c1 = " ; printnum(ctemp1);cout << endl;
        cout << "c2 = " ; printnum(ctemp2);cout << endl;
        cout << "c3 = " ; printnum(ctemp3);cout << endl;
#endif
	diff = abs(Ctemp3 - ctemp3);
#if defined VERBOSE_TEST
        cout << "residue=C1-C2-C3" << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
        cout << "residue=C3-c3 " << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
	cout << "DIFF = "; printnum(diff); cout << endl;  cout << endl;
#endif
	if (abs(diff) > EPSILON) {
	    flag = 1;
	}
    }
    if (flag) {
	printf("Complex - Complex SUB test failed\n");
	exit(1);
    } else {
	printf("Complex - Complex SUB test passed\n");
    }
}

void
mpc_sub_test2()
{
    printf("Complex - Real test \n");
    COMPLEX Ctemp1;
    REAL Ftemp2;
    COMPLEX Ctemp3;

    REAL Ftemp;
    REAL diff;

    complex < double >ctemp1;
    double ftemp2;
    complex < double >ctemp3;

    int flag = 0;

    for (int i = 0; i < MAX_ITER; i++) {
	Ctemp1 = mpc_randomnumber();
	Ftemp2 = mpf_randomnumber();

	ctemp1.real() = cast2double(Ctemp1.re);
	ctemp1.imag() = cast2double(Ctemp1.im);
	ftemp2 = cast2double(Ftemp2);
	Ctemp3 = Ctemp1 - Ftemp2;
	ctemp3 = ctemp1 - ftemp2;
#if defined VERBOSE_TEST
        cout << "C1 = " ; printnum(Ctemp1);cout << endl;
        cout << "F2 = " ; printnum(Ftemp2);cout << endl;
        cout << "C3 = " ; printnum(Ctemp3);cout << endl;
        cout << endl;
        cout << "c1 = " ; printnum(ctemp1);cout << endl;
        cout << "f2 = " ; printnum(ftemp2);cout << endl;
        cout << "c3 = " ; printnum(ctemp3);cout << endl;
#endif
	diff = abs(Ctemp3 - ctemp3);
#if defined VERBOSE_TEST
        cout << "residue=C1-F2-C3" << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
        cout << "residue=C3-c3 " << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
	cout << "DIFF = "; printnum(diff); cout << endl;  cout << endl;
#endif
	if (abs(diff) > EPSILON) {
	    flag = 1;
	}
    }
    if (flag) {
	printf("Complex - Real test failed\n");
	exit(1);
    } else {
	printf("Complex - Real test passed\n");
    }
}

void
mpc_sub_test3()
{
    printf("Real - Complex test \n");
    REAL Ftemp1;
    COMPLEX Ctemp2;
    COMPLEX Ctemp3;

    REAL diff;

    double ftemp1;
    complex < double >ctemp2;
    complex < double >ctemp3;

    int flag = 0;

    for (int i = 0; i < MAX_ITER; i++) {
	Ctemp2 = mpc_randomnumber();
	Ftemp1 = mpf_randomnumber();

	ctemp2.real() = cast2double(Ctemp2.re);
	ctemp2.imag() = cast2double(Ctemp2.im);
	ftemp1 = cast2double(Ftemp1);
	Ctemp3 = Ftemp1 - Ctemp2;
	ctemp3 = ftemp1 - ctemp2;
#if defined VERBOSE_TEST
        cout << "F1 = " ; printnum(Ftemp1);cout << endl;
        cout << "C2 = " ; printnum(Ctemp2);cout << endl;
        cout << "C3 = " ; printnum(Ctemp3);cout << endl;
        cout << endl;
        cout << "f1 = " ; printnum(ftemp1);cout << endl;
        cout << "c2 = " ; printnum(ctemp2);cout << endl;
        cout << "c3 = " ; printnum(ctemp3);cout << endl;
#endif
	diff = abs(Ctemp3 - ctemp3);
#if defined VERBOSE_TEST
        cout << "residue=F1-C2-C3" << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
        cout << "residue=C3-c3 " << endl;
        cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
	cout << "DIFF = "; printnum(diff); cout << endl;  cout << endl;
#endif
	if (abs(diff) > EPSILON) {
	    flag = 1;
	}
    }
    if (flag) {
	printf("Real - Complex test failed\n");
	exit(1);
    } else {
	printf("Real - Complex test passed\n");
    }
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    mpc_subst_test1();
    mpc_abs_test();
    mpc_add_test1();
    mpc_add_test2();
    mpc_add_test3();
    mpc_mul_test1();
    mpc_mul_test2();
    mpc_mul_test3();
    mpc_div_test1();
    mpc_div_test2();
    mpc_div_test3();
    mpc_sub_test1();
    mpc_sub_test2();
    mpc_sub_test3();
    printf("All tests passed succesfully\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif
  
  return(0);
}
