/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Clansy.debug.cpp,v 1.1 2009/12/21 03:34:05 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <mlapack.h>
#include <mpack_debug.h>

#include <blas.h>
#include <lapack.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_N     0
#define MAX_N     50
#define MAX_LDA   60
#define MAX_ITER  10

void Clansy_test2(const char *norm, const char *uplo);

void
Clansy_test2(const char *norm, const char *uplo)
{
    int errorflag = FALSE;
    int j = 0;
    REAL Clansy_ret;
    double zlansy_ret;

    for (int n = MIN_N; n < MAX_N; n++) {
	for (int lda = max(n, 1); lda < MAX_LDA; lda++) {
#if defined VERBOSE_TEST
	printf("n:%d lda %d, uplo %s, norm %s\n", n, lda, uplo, norm);
#endif
	    COMPLEX *A = new COMPLEX[matlen(lda, n)];
	    REAL *work = new REAL[max(1, n)];
	    complex<double> *Ad = new complex<double>[matlen(lda, n)];
	    double *workd = new double[max(1, n)];
	    j = 0;
	    while (j < MAX_ITER) {
		set_random_vector(A, Ad, matlen(lda, n));
		Clansy_ret = Clansy(norm, uplo, n, A, lda, work);
		zlansy_ret = zlansy_f77(norm, uplo, &n, Ad, &lda, workd);
		double diff = num_diff(Clansy_ret, zlansy_ret);
		if (fabs(diff) > EPSILON) {
		    printf("n:%d lda %d, uplo %s, norm %s\n", n, lda, uplo,
			norm);
		    printf("error %.16e!!\n", diff);
		    errorflag = TRUE;
		}
		j++;
	    }
	    delete[]A;
	    delete[]Ad;
	    delete[]work;
	    delete[]workd;
	}
	if (errorflag == TRUE) {
	    printf("Clansy test failed...\n");
	    exit(1);
	}
    }
}

void
Clansy_test(void)
{
    Clansy_test2("M", "U");
    Clansy_test2("m", "U");
    Clansy_test2("1", "U");
    Clansy_test2("O", "U");
    Clansy_test2("o", "U");
    Clansy_test2("I", "U");
    Clansy_test2("i", "U");
    Clansy_test2("F", "U");
    Clansy_test2("f", "U");
    Clansy_test2("E", "U");
    Clansy_test2("e", "U");

    Clansy_test2("M", "L");
    Clansy_test2("m", "L");
    Clansy_test2("1", "L");
    Clansy_test2("O", "L");
    Clansy_test2("o", "L");
    Clansy_test2("I", "L");
    Clansy_test2("i", "L");
    Clansy_test2("F", "L");
    Clansy_test2("f", "L");
    Clansy_test2("E", "L");
    Clansy_test2("e", "L");
}
int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Clansy_test();
    printf("Clansy test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
