/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Clarf.debug.cpp,v 1.1 2009/12/22 10:14:15 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <mlapack.h>
#include <mpack_debug.h>

#include <blas.h>
#include <lapack.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_N 2
#define MAX_N 20
#define MIN_M 2
#define MAX_M 20
#define MIN_LDC 2
#define MAX_LDC 20
#define MAX_ITER 3
#define MIN_INCV  1
#define MAX_INCV  3

void
Clarf_test2(const char *side)
{
    int errorflag = FALSE;
    int dimv;
    int n, m, ldc, incv, iter;
    COMPLEX tau;
    COMPLEX *C, *v, *work;
    complex<double> taud;
    complex<double> *Cd, *vd, *workd;
    double diff;

    for (incv = MIN_INCV; incv <= MAX_INCV; incv++) {
	if (incv == 0)
	    continue;
	for (n = MIN_N; n <= MAX_N; n++) {
	    for (m = MIN_M; m <= MAX_M; m++) {
		for (ldc = max(1, m); ldc <= MAX_LDC; ldc++) {

		    if (Mlsame(side, "L"))
			dimv = (1 + (m - 1) * abs(incv));
		    if (Mlsame(side, "R"))
			dimv = (1 + (n - 1) * abs(incv));

		    C = new COMPLEX[matlen(ldc, n)];
		    v = new COMPLEX[dimv];
		    if (Mlsame(side, "L")) work = new COMPLEX[n];
		    if (Mlsame(side, "R")) work = new COMPLEX[m];

		    Cd = new complex<double>[matlen(ldc, n)];
		    vd = new complex<double>[dimv];

		    if (Mlsame(side, "L"))
			workd = new complex<double>[n];

		    if (Mlsame(side, "R"))
			workd = new complex<double>[m];
#if defined VERBOSE_TEST
		    printf("# m: %d, n: %d, incv: %d, ldc: %d, side %s\n", m,
			n, incv, ldc, side); 
#endif
		    for (iter = 0; iter < MAX_ITER; iter++) {
			set_random_vector(C, Cd, matlen(ldc, n));
			set_random_vector(v, vd, dimv);
			set_random_number(&tau, &taud);

			Clarf(side, m, n, v, incv, tau, C, ldc, work);
			zlarf_f77(side, &m, &n, vd, &incv, &taud, Cd, &ldc,
			    workd);

			diff = infnorm(C, Cd, matlen(ldc, n), 1);
			if (fabs(diff) > EPSILON) {
			    printf("#error %.16e!!\n", diff);
			    printf("# tau != 0.0\n");
			    printf
				("# m: %d, n: %d, incv: %d, ldc: %d, side %s\n",
				m, n, incv, ldc, side);
			    errorflag = TRUE;
			}
		    }

//tau = 0 case
		    for (iter = 0; iter < MAX_ITER; iter++) {
			set_random_vector(C, Cd, matlen(ldc, n));
			set_random_vector(v, vd, dimv);
			tau = 0.0;
			taud = 0.0;

			Clarf(side, m, n, v, incv, tau, C, ldc, work);
			zlarf_f77(side, &m, &n, vd, &incv, &taud, Cd, &ldc,
			    workd);

			diff = infnorm(C, Cd, matlen(ldc, n), 1);
			if (fabs(diff) > EPSILON) {
			    printf("# tau == 0.0\n");
			    printf
				("# m: %d, n: %d, incv: %d, ldc: %d, side %s\n",
				m, n, incv, ldc, side);
			    printf("#error %.16e!!\n", diff);
			}
		    }
		    delete[]vd;
		    delete[]Cd;
		    delete[]v;
		    delete[]C;

		}
	    }

	}
    }
    if (errorflag == TRUE) {
	printf("Clarf test failed...\n");
        exit(1);
    }
}

void
Clarf_test()
{
    Clarf_test2("L");
    Clarf_test2("R");
}
int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Clarf_test();
    printf("Clarf test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
