/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Csteqr.debug.cpp,v 1.1 2009/12/22 05:12:30 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <mlapack.h>
#include <mpack_debug.h>

#include <blas.h>
#include <lapack.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_N      0
#define MAX_N     40
#define MAX_LDZ   40
#define MAX_ITER   5

void Csteqr_test2(const char *uplo);

void
Csteqr_test2(const char *compz)
{
    int errorflag = FALSE;
    int j = 0;
    int infod, ldzd, nd;
    INTEGER info; 
    double diff;

    for (INTEGER n = MIN_N; n < MAX_N; n++) {
	for (INTEGER ldz = max(n, (INTEGER)1); ldz < MAX_LDZ; ldz++) {
	    COMPLEX *Z = new COMPLEX[matlen(ldz, n)];
	    REAL *D = new REAL[veclen(n, 1)];
	    REAL *E = new REAL[veclen(n - 1, 1)];
	    REAL *work = new REAL[max((INTEGER)1, 2 * n - 2)];

	    complex<double> *Zd = new complex<double>[matlen(ldz, n)];
	    double *Dd = new double[veclen(n, 1)];
	    double *Ed = new double[veclen(n - 1, 1)];
	    double *workd = new double[max((INTEGER)1, 2 * n - 2)];
#if defined VERBOSE_TEST
	    printf("# compz %s, n:%d ldz %d\n", compz, (int)n, (int)ldz);
#endif
	    j = 0;
	    while (j < MAX_ITER) {
		set_random_vector(Z, Zd, matlen(ldz, n));
		set_random_vector(D, Dd, veclen(n, 1));
		set_random_vector(E, Ed, veclen(n - 1, 1));
		set_random_vector(work, workd, max((INTEGER)1, 2 * n - 2));
		Csteqr(compz, n, D, E, Z, ldz, work, &info);
                nd = n; ldzd = ldz;
		zsteqr_f77(compz, &nd, Dd, Ed, Zd, &ldzd, workd, &infod);
		if (info < 0) {
		    printf("info %d error\n", -(int)info);
		    errorflag = TRUE;
		}
		if (info != infod) {
		    printf("info differ! %d, %d\n", (int)info, infod);
		    errorflag = TRUE;
		}
	        if (Mlsame (compz, "I")) {
/* not yet implemented */
/* eigenvectors of tridiagonal matrix */
		}
	        if (Mlsame (compz, "V")) {
		  /* not to be implemented */
		  /* this part can be qa'ed by Rsyev.cpp */
		 }
		diff = infnorm(D, Dd, veclen(n, 1), 1);
		if (fabs(diff) > EPSILON) {
		    printf("Eigenvalue %.16e!!\n", diff);
		    errorflag = TRUE;
		}
		j++;
	    }
	    delete[]Z;
	    delete[]D;
	    delete[]E;
	    delete[]work;
	    delete[]Zd;
	    delete[]Dd;
	    delete[]Ed;
	    delete[]workd;
	}
	if (errorflag == TRUE) {
	    printf("Csteqr test failed...\n");
	    exit(1);
	}
    }
}

void
Csteqr_test(void)
{
    Csteqr_test2("V");
//    Csteqr_test2("I");
    Csteqr_test2("N");
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Csteqr_test();
    printf("Csteqr test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}

