/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Mutils.debug.cpp,v 1.4 2009/09/25 04:00:39 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
/*
Copyright (c) 1992-2007 The University of Tennessee.  All rights reserved.

$COPYRIGHT$

Additional copyrights may follow

$HEADER$

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer. 
  
- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer listed
  in this license in the documentation and/or other materials
  provided with the distribution.
  
- Neither the name of the copyright holders nor the names of its
  contributors may be used to endorse or promote products derived from
  this software without specific prior written permission.
  
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT  
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT  
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/

#include <mblas.h>
#include <mlapack.h>
#include <blas.h>
#include <mpack_debug.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define ITERATION 100

//ln2 test
int Mutils_test_log2()
{
    int errorflag = TRUE;
    REAL a, b;
    double ad, bd;

    printf("approx ln2 test\n");
    for (int i = 0; i < ITERATION; i++) {
	set_random_number(&a, &ad);
	a = abs(a);
	ad = fabs(ad);
	b = mpf_approx_log2(a);
	bd = log10(ad) / log10(2);
#if defined VERBOSE_TEST
       cout << "a=       " ; printnum(a);  cout << endl;
       cout << "ln2a=    " ; printnum(b);  cout << endl;
       cout << "ad=      " ; printnum(ad); cout << endl;
       cout << "ln2ad=   " ; printnum(bd); cout << endl;
       cout << "residue=ln2a-ln2ad " << endl;
       cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
       cout << endl;
#endif
	if (num_diff(b, bd) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error in log2\n");
            exit(1);
	}
    }
    return errorflag;
}

//log10 test
int Mutils_test_log10()
{
    int errorflag = TRUE;
    REAL a, b;
    double ad, bd;

    printf("approx log10 test\n");
    for (int i = 0; i < ITERATION; i++) {
	set_random_number(&a, &ad);
	a = abs(a);
	ad = fabs(ad);
	b = mpf_approx_log10(a);
	bd = log10(ad);
#if defined VERBOSE_TEST
       cout << "a=       " ; printnum(a);  cout << endl;
       cout << "log10a=  " ; printnum(b);  cout << endl;
       cout << "ad=      " ; printnum(ad); cout << endl;
       cout << "log10ad= " ; printnum(bd); cout << endl;
       cout << "residue=log10a-log10ad" << endl;
       cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
       cout << endl;
#endif
	if (num_diff(b, bd) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error in log10\n");
            exit(1);
	}
    }
    return errorflag;
}

//pow test
int Mutils_test_pow()
{
    int errorflag = TRUE;
    REAL x, y, z;
    double xd, yd, zd;

    printf("approx log10 test\n");
    for (int i = 0; i < ITERATION; i++) {
	set_random_number(&x, &xd);
	set_random_number(&y, &yd);
	x  = abs(x);  y  = abs(y);
	xd = abs(xd); yd = abs(yd);
        z  = mpf_approx_pow (x,y);
        zd = pow (xd,yd);
#if defined VERBOSE_TEST
       cout << "x=       " ; printnum(x);  cout << endl;
       cout << "y=       " ; printnum(y);  cout << endl;
       cout << "z=       " ; printnum(z);  cout << endl;
       cout << "xd=      " ; printnum(xd); cout << endl;
       cout << "yd=      " ; printnum(yd); cout << endl;
       cout << "zd=      " ; printnum(zd); cout << endl;
       cout << "residue=z-zd " ; cout << endl;
       cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
       cout << endl;
#endif
	if (num_diff(z, zd) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error in pow\n");
            exit(1);
	}
    }
    return errorflag;
}

//ln test
int Mutils_test_log()
{
    int errorflag = TRUE;
    REAL a, b;
    double ad, bd;

    printf("approx log test\n");
    for (int i = 0; i < ITERATION; i++) {
	set_random_number(&a, &ad);
	a = abs(a);
	ad = fabs(ad);
	b = mpf_approx_log(a);
	bd = log(ad);
#if defined VERBOSE_TEST
       cout << "a=       " ; printnum(a);  cout << endl;
       cout << "lna=     " ; printnum(b);  cout << endl;
       cout << "ad=      " ; printnum(ad); cout << endl;
       cout << "lnad=    " ; printnum(bd); cout << endl;
       cout << "residue=lna-lnad" << endl;
       cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
       cout << endl;
#endif
	if (num_diff(b, bd) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error log\n");
            exit(1);
	}
    }
    return errorflag;
}

//sin test
int Mutils_test_sin()
{
    int errorflag = TRUE;
    REAL a, b;
    double ad, bd;

    printf("approx sin test\n");
    for (int i = 0; i < ITERATION; i++) {
	set_random_number(&a, &ad);
	b = mpf_approx_sin(a);
	bd = sin(ad);
#if defined VERBOSE_TEST
       cout << "a=       " ; printnum(a);  cout << endl;
       cout << "s=       " ; printnum(b);  cout << endl;
       cout << "ad=      " ; printnum(ad); cout << endl;
       cout << "sd=      " ; printnum(bd); cout << endl;
       cout << "residue=s-sd" << endl;
       cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
       cout << endl;
#endif
	if (num_diff(b, bd) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error sin\n");
            exit(1);
	}
    }
    return errorflag;
}

//cos test
int Mutils_test_cos()
{
    int errorflag = TRUE;
    REAL a, b;
    double ad, bd;

    printf("approx cos test\n");
    for (int i = 0; i < ITERATION; i++) {
	set_random_number(&a, &ad);
	b = mpf_approx_cos(a);
	bd = cos(ad);
#if defined VERBOSE_TEST
       cout << "a=       " ; printnum(a);  cout << endl;
       cout << "c=       " ; printnum(b);  cout << endl;
       cout << "ad=      " ; printnum(ad); cout << endl;
       cout << "cd=      " ; printnum(bd); cout << endl;
       cout << "residue=c-cd" << endl;
       cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
       cout << endl;
#endif
	if (num_diff(b, bd) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error cos\n");
            exit(1);
	}
    }
    return errorflag;
}

//exp test
int Mutils_test_exp()
{
    int errorflag = TRUE;
    REAL a, b;
    double ad, bd;

    printf("approx exp test\n");
    for (int i = 0; i < ITERATION; i++) {
	set_random_number(&a, &ad);
	b = mpf_approx_exp(a);
	bd = exp(ad);
#if defined VERBOSE_TEST
       cout << "a=       " ; printnum(a);  cout << endl;
       cout << "expa=    " ; printnum(b);  cout << endl;
       cout << "ad=      " ; printnum(ad); cout << endl;
       cout << "expad=   " ; printnum(bd); cout << endl;
       cout << "residue=expa-expad" << endl;
       cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
       cout << endl;
#endif
	if (num_diff(b, bd) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error in exp\n");
            exit(1);
	}
    }
    return errorflag;
}

int
Mutils_test_pi()
{
    int errorflag = TRUE;
    REAL p;
    double pd;

    printf("approx constant pi test\n");
	p = mpf_approx_pi();
	pd = M_PI;
#if defined VERBOSE_TEST
       cout << "p=       " ; printnum(p);  cout << endl;
       cout << "pd=      " ; printnum(pd); cout << endl;
       cout << "residue=p-pd" << endl;
       cout << "if(abs(residue)< " << EPSILON << ") printf \"ok\\n\"; else printf \"ng\\n\"; endif"  << endl;
       cout << endl;
#endif
	if (num_diff(p, pd) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error in constant pi\n");
            exit(1);
	}
    return errorflag;
}

int
Mutils_test_Msign()
{
    int errorflag = TRUE;
    REAL mtemp;

    printf("Msign test\n");
    mtemp = Msign(1.0, -1.0);
    if (mtemp != -1.0) {
	printf("msign error\n");
	errorflag = FALSE;
    }

    mtemp = Msign(-1.0, 1.0);
    if (mtemp != 1.0) {
	printf("msign error\n");
	errorflag = FALSE;
    }

    mtemp = Msign(-1.0, 0.0);
    if (mtemp != 1.0) {
	printf("msign error\n");
	errorflag = FALSE;
    }

    mtemp = Msign(1.0, 0.0);
    if (mtemp != 1.0) {
	printf("msign error\n");
	errorflag = FALSE;
    }

    mtemp = Msign(0.0, 0.0);
    if (mtemp != 0.0) {
	printf("msign error\n");
	errorflag = FALSE;
    }

    if (errorflag == FALSE) {
      printf("Error in Msign\n");
      exit(1);
    }
    return errorflag;
}


void
Mutils_test()
{
    Mutils_test_log2();
    Mutils_test_log10();
    Mutils_test_log();
    Mutils_test_Msign();
    Mutils_test_pow();
    Mutils_test_sin();
    Mutils_test_cos();
    Mutils_test_exp();
    Mutils_test_pi();
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Mutils_test();
    printf("Mutils test passed succesfully\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
