/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Rorgtr.debug.cpp,v 1.4 2009/12/25 02:07:26 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <mlapack.h>
#include <mpack_debug.h>

#include <blas.h>
#include <lapack.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_N 10
#define MAX_N 30
#define MIN_M 10
#define MAX_M 30
#define MIN_K 10
#define MAX_K 30
#define MIN_LDA 10
#define MAX_LDA 30
#define MAX_ITER 30

void
Rorgtr_test2(const char *uplo)
{
    int errorflag = FALSE;
    int iter;
    int n, lda, infod;
    int lwork, worksized, worksize;
    mpackint info;

    for (n = MIN_N; n <= MAX_N; n++) {
	for (lda = max(1, n); lda <= MAX_LDA; lda++) {
#if defined VERBOSE_TEST
	    printf("# uplo %s, n %d, lda %d\n", uplo, n, lda);
#endif
	    REAL *A = new REAL[matlen(lda, n)];
	    REAL *tau = new REAL[veclen(n - 1, 1)];
	    REAL *work = new REAL[veclen(n - 1, 1) * 1024];

	    double *Ad = new double[matlen(lda, n)];
	    double *taud = new double[veclen(n - 1, 1)];
	    double *workd = new double[veclen(n - 1, 1) * 1024];

//these workspace query might not be the same value.
	    lwork = -1;
	    dorgtr_f77(uplo, &n, Ad, &lda, taud, workd, &lwork, &infod);
	    Rorgtr(uplo, n, A, lda, tau, work, lwork, &info);
  	    worksize = (int)(cast2double(work[0]));
	    worksized = (int)workd[0];
#if defined VERBOSE_TEST
            printf ("optimized worksize by dorgtr %d : by Rorgtr %d.\n", worksized, worksize);
#endif
#ifdef DUMMY
//comparison of workspace is nonsense...
            if(worksize!=worksized) printf("error in worksize\n");
#endif
	    for (iter = 0; iter < MAX_ITER; iter++) {
		set_random_vector(A, Ad, matlen(lda, n));
		set_random_vector(tau, taud, veclen(n - 1, 1));
		set_random_vector(work, workd, veclen(n - 1, 1) * 1024);

		lwork = worksized;
		dorgtr_f77(uplo, &n, Ad, &lda, taud, workd, &lwork, &infod);
		Rorgtr(uplo, n, A, lda, tau, work, lwork, &info);

		double diff;

		diff = infnorm(A, Ad, matlen(lda, n), 1);
		if (fabs(diff) > EPSILON) {
		    printf("error in 'A' %.16e!!\n", diff);
		    errorflag = TRUE;
		    exit(1);
		}
		diff = infnorm(tau, taud, veclen(n - 1, 1), 1);
		if (fabs(diff) > EPSILON) {
		    printf("error in 'W' %.16e!!\n", diff);
		    errorflag = TRUE;
		    exit(1);
		}
#ifdef DUMMY
//comparison of workspace is nonsense...
		diff = infnorm(work, workd, veclen(n - 1, 1), 1);
		if (fabs(diff) > EPSILON) {
		    printf("error in 'work' %.16e!!\n", diff);
		    errorflag = TRUE;
		    exit(1);
		}
#endif
	    }
	    delete[]taud;
	    delete[]workd;
	    delete[]Ad;
	    delete[]tau;
	    delete[]work;
	    delete[]A;
	}
    }
    if (errorflag == TRUE) {
	printf("Rorgtr test failed...\n");
	exit(-1);
    }
}
void
Rorgtr_test()
{
    Rorgtr_test2("L");
    Rorgtr_test2("U");
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Rorgtr_test();
    printf("Rorgtr test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
