/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Rsyev.debug.cpp,v 1.6 2009/12/25 03:07:54 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <mlapack.h>
#include <mpack_debug.h>

#include <blas.h>
#include <lapack.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_N      0
#define MAX_N     30
#define MAX_LDA   30
#define MAX_ITER  10

void Rsyev_test2(const char *jobz, const char *uplo);

void
Rsyev_test2(const char *jobz, const char *uplo)
{
    int errorflag = FALSE;
    int j = 0;
    mpackint info, lwork;
    int infod, lworkd;
    double diff;

    for (int n = MIN_N; n < MAX_N; n++) {
	for (int lda = max(n, 1); lda < MAX_LDA; lda++) {
	    REAL *A = new REAL[matlen(lda, n)];
	    REAL *w = new REAL[veclen(n, 1)];
	    double *Ad = new double[matlen(lda, n)];
	    double *wd = new double[veclen(n, 1)];
#if defined VERBOSE_TEST
	    printf("# jobz %s, uplo %s,  n:%d lda %d\n", jobz, uplo, n, lda);
#endif
	    REAL *work = new REAL[1];
	    double *workd = new double[1];

//these workspace query might not be the same value.
	    lwork = -1;
	    lworkd = -1;
	    Rsyev(jobz, uplo, n, A, lda, w, work, lwork, &info);
	    dsyev_f77(jobz, uplo, &n, Ad, &lda, wd, workd, &lworkd, &infod);
  	    lwork = (int)(cast2double(work[0]));
	    lworkd = (int)workd[0];
#if defined VERBOSE_TEST
            printf ("optimized worksize by dsyev %d : by Rsyev %d.\n", lworkd, (int)lwork);
#endif
#ifdef DUMMY
//comparison of workspace is nonsense...
            if(worksize!=worksized) printf("error in worksize\n");
#endif
	    delete[]work;
	    delete[]workd;
            work = new REAL[max(1, (int)lwork)];
            workd = new double[max(1, lworkd)];

	    j = 0;
	    while (j < MAX_ITER) {
		set_random_vector(A, Ad, matlen(lda, n));
		set_random_vector(w, wd, veclen(n, 1));
		set_random_vector(work, workd, veclen(lwork, 1));

		Rsyev(jobz, uplo, n, A, lda, w, work, lwork, &info);
		dsyev_f77(jobz, uplo, &n, Ad, &lda, wd, workd, &lworkd, &infod);

		if (info < 0) {
		    printf("info %d error\n", -(int)info);
		    errorflag = TRUE;
		}
		if (info != infod) {
		    printf("info differ! %d, %d\n", (int)info, infod);
		    errorflag = TRUE;
		}
/*
		diff = infnorm(A, Ad, matlen(lda, n), 1);
		if (fabs(diff) > EPSILON3) {
		    printf("Ret mat A error %e!!\n", diff);
printf("A =");      printmat(n, n, A,lda); printf("\n");
printf("Ad=");      printmat(n,n, Ad,lda); printf("\n");
		    errorflag = TRUE;
		}
*/
		diff = infnorm(w, wd, veclen(n, 1), 1);
		if (fabs(diff) > EPSILON) {
		    printf("Eigenvalue %.16e!!\n", diff);
		    errorflag = TRUE;
		}
		j++;
	    }
	    delete[]work;
	    delete[]A;
	    delete[]w;

	    delete[]workd;
	    delete[]Ad;
	    delete[]wd;
	}
	if (errorflag == TRUE) {
	    printf("Rsyev test failed...\n");
	    exit(1);
	}
    }
}

void
Rsyev_test(void)
{
    Rsyev_test2("N", "U");
    Rsyev_test2("N", "L");
    Rsyev_test2("V", "U");
    Rsyev_test2("V", "L");
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Rsyev_test();
    printf("Rsyev test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
