/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Rsytd2.debug.cpp,v 1.4 2009/09/25 04:00:39 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <mlapack.h>
#include <mpack_debug.h>

#include <blas.h>
#include <lapack.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_N      0
#define MAX_N     50
#define MAX_LDA   50
#define MAX_ITER  1

void Rsytd2_test2(const char *uplo);

void
Rsytd2_test2(const char *uplo)
{
    int errorflag = FALSE;
    int j = 0;
    int infod;
    double diff;
    mpackint info;

    for (int n = MIN_N; n < MAX_N; n++) {
	for (int lda = max(n, 1); lda < MAX_LDA; lda++) {

	    REAL *A = new REAL[matlen(lda, n)];
	    REAL *d = new REAL[veclen(n, 1)];
	    REAL *e = new REAL[veclen(n - 1, 1)];
	    REAL *tau = new REAL[veclen(n - 1, 1)];

	    double *Ad = new double[matlen(lda, n)];
	    double *dd = new double[veclen(n, 1)];
	    double *ed = new double[veclen(n - 1, 1)];
	    double *taud = new double[veclen(n - 1, 1)];
#if defined VERBOSE_TEST
	    printf("#uplo %s, n:%d lda %d\n", uplo, n, lda);
#endif
	    j = 0;
	    while (j < MAX_ITER) {
		set_random_vector(A, Ad, matlen(lda, n));
		set_random_vector(d, dd, veclen(n, 1));
		set_random_vector(e, ed, veclen(n - 1, 1));
		set_random_vector(tau, taud, veclen(n - 1, 1));

		Rsytd2(uplo, n, A, lda, d, e, tau, &info);
		dsytd2_f77(uplo, &n, Ad, &lda, dd, ed, taud, &infod);
		if (info < 0) {
		    printf("info %d error\n", -(int)info);
		    errorflag = TRUE;
		}
		if (info != infod) {
		    printf("info differ! %d, %d\n", (int)info, infod);
		    errorflag = TRUE;
		}
		diff = infnorm(A, Ad, matlen(lda, n), 1);
		if (fabs(diff) > EPSILON) {
		    printf("Ret mat A error %e!!\n", diff);
		    errorflag = TRUE;
		}
		diff = infnorm(d, dd, veclen(n, 1), 1);
		if (fabs(diff) > EPSILON) {
		    printf("diagonal part of tridiagonal mat error%e!!\n",
			diff);
		    errorflag = TRUE;
		}

		diff = infnorm(e, ed, veclen(n - 1, 1), 1);
		if (fabs(diff) > EPSILON) {
		    printf("off-diagonal part of tridiagonal mat error%e!!\n",
			diff);
		    errorflag = TRUE;
		}
		j++;
	    }
	    delete[]taud;
	    delete[]ed;
	    delete[]dd;
	    delete[]Ad;
	    delete[]tau;
	    delete[]e;
	    delete[]d;
	    delete[]A;
	}
	if (errorflag == TRUE) {
	    printf("Rsytd2 test failed...\n");
	    exit(1);
	}
    }
}

void
Rsytd2_test(void)
{
    Rsytd2_test2("U");
    Rsytd2_test2("L");
}
int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Rsytd2_test();
    printf("Rsytd2 test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
